"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

require("core-js/modules/web.dom.iterable");

var _path = _interopRequireDefault(require("path"));

var _Disk = _interopRequireDefault(require("@polkadot/db/Disk"));

var _Memory = _interopRequireDefault(require("@polkadot/db/Memory"));

var _trieDb = _interopRequireDefault(require("@polkadot/trie-db"));

var _logger = _interopRequireDefault(require("@polkadot/util/logger"));

var _toHex = _interopRequireDefault(require("@polkadot/util/u8a/toHex"));

var _block = _interopRequireDefault(require("./block"));

var _state = _interopRequireDefault(require("./state"));

// Copyright 2017-2018 @polkadot/client-chains authors & contributors
// This software may be modified and distributed under the terms
// of the ISC license. See the LICENSE file for details.
const SPINNER = ['|', '/', '-', '\\'];
const PREPEND = '                                     ';
const l = (0, _logger.default)('db');

class Dbs {
  constructor({
    db
  }, chain) {
    this.blocks = void 0;
    this.state = void 0;
    this.stateBackup = void 0;
    this.basePath = void 0;
    this.basePath = db.type === 'disk' ? _path.default.join(db.path, 'chains', chain.id, (0, _toHex.default)(chain.genesisRoot)) : ''; // NOTE blocks compress very well

    this.blocks = (0, _block.default)(this.createBackingDb(db, 'block.db', true)); // NOTE state RLP does not compress well here

    this.state = (0, _state.default)(new _trieDb.default(this.createBackingDb(db, 'state.db', false)));
    this.stateBackup = db.snapshot ? (0, _state.default)(new _trieDb.default(this.createBackingDb(db, 'state.db.snapshot', false))) : null;

    if (db.compact) {
      this.maintain('block', this.blocks.db);
      this.maintain('state', this.state.db);
    }

    this.blocks.db.open();
    this.state.db.open();
  }

  createBackingDb({
    type
  }, name, isCompressed) {
    return type === 'disk' ? new _Disk.default(this.basePath, name, {
      isCompressed
    }) : new _Memory.default();
  }

  maintain(name, db) {
    l.log(`compacting ${name} database`);
    db.maintain(this.createProgress());
  }

  snapshotState() {
    if (!this.stateBackup) {
      return;
    }

    this.stateBackup.db.open();
    this.state.db.snapshot(this.stateBackup.db, this.createProgress());
    this.state.db.close();
    this.state.db.rename(this.basePath, `state.db.backup-${Date.now()}`);
    this.stateBackup.db.close();
    this.stateBackup.db.rename(this.basePath, 'state.db');
    this.stateBackup.db.open();
    this.state.db = this.stateBackup.db;
  }

  createProgress() {
    let lastUpdate = 0;
    let spin = 0;
    return progress => {
      const now = Date.now();

      if (now - lastUpdate > 200) {
        const percent = `      ${progress.percent.toFixed(2)}`.slice(-6);
        const keys = progress.keys > 9999 ? `${(progress.keys / 1000).toFixed(2)}k` : progress.keys;
        process.stdout.write(`${PREPEND}${SPINNER[spin % SPINNER.length]} ${percent}%, ${keys} keys\r`);
        lastUpdate = now;
        spin++;
      }
    };
  }

}

exports.default = Dbs;