"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createState;

var _storage = _interopRequireDefault(require("@polkadot/storage"));

var _account = _interopRequireDefault(require("../db/account"));

var _arrayAccount = _interopRequireDefault(require("../db/arrayAccount"));

var _arrayU8a = _interopRequireDefault(require("../db/arrayU8a"));

var _bn = _interopRequireDefault(require("../db/bn"));

var _bool = _interopRequireDefault(require("../db/bool"));

var _u8a = _interopRequireDefault(require("../db/u8a"));

// Copyright 2017-2018 @polkadot/client-db authors & contributors
// This software may be modified and distributed under the terms
// of the ISC license. See the LICENSE file for details.
const BALANCE_SIZE = 128;
const BLOCKNUM_SIZE = 64;

const consensus = (db, {
  public: {
    authorityAt,
    authorityCount,
    code
  }
}) => ({
  authorityAt: (0, _account.default)(db, authorityAt),
  authorityCount: (0, _bn.default)(db, authorityCount, 32),
  code: (0, _u8a.default)(db, code)
});

const council = (db, {
  public: {
    activeCouncil,
    candidacyBond,
    carryCount,
    desiredSeats,
    inactiveGracePeriod,
    presentationDuration,
    presentSlashPerVoter,
    termDuration,
    votingBond,
    votingPeriod
  }
}) => ({
  activeCouncil: (0, _arrayU8a.default)(db, activeCouncil),
  candidacyBond: (0, _bn.default)(db, candidacyBond, BALANCE_SIZE),
  carryCount: (0, _bn.default)(db, carryCount, 32),
  desiredSeats: (0, _bn.default)(db, desiredSeats, 32),
  inactiveGracePeriod: (0, _bn.default)(db, inactiveGracePeriod, 32),
  presentationDuration: (0, _bn.default)(db, presentationDuration, BLOCKNUM_SIZE),
  presentSlashPerVoter: (0, _bn.default)(db, presentSlashPerVoter, BALANCE_SIZE),
  termDuration: (0, _bn.default)(db, termDuration, BLOCKNUM_SIZE),
  votingBond: (0, _bn.default)(db, votingBond, BALANCE_SIZE),
  votingPeriod: (0, _bn.default)(db, votingPeriod, BLOCKNUM_SIZE)
});

const councilVoting = (db, {
  public: {
    cooloffPeriod,
    votingPeriod
  }
}) => ({
  cooloffPeriod: (0, _bn.default)(db, cooloffPeriod, BLOCKNUM_SIZE),
  votingPeriod: (0, _bn.default)(db, votingPeriod, BLOCKNUM_SIZE)
});

const democracy = (db, {
  public: {
    launchPeriod,
    minimumDeposit,
    votingPeriod
  }
}) => ({
  launchPeriod: (0, _bn.default)(db, launchPeriod, BLOCKNUM_SIZE),
  minimumDeposit: (0, _bn.default)(db, minimumDeposit, BALANCE_SIZE),
  votingPeriod: (0, _bn.default)(db, votingPeriod, BLOCKNUM_SIZE)
});

const parachains = (db, {
  public: {
    didUpdate
  }
}) => ({
  didUpdate: (0, _bool.default)(db, didUpdate)
});

const session = (db, {
  public: {
    length,
    validators
  }
}) => ({
  length: (0, _bn.default)(db, length, BLOCKNUM_SIZE),
  validators: (0, _arrayAccount.default)(db, validators)
});

const staking = (db, {
  public: {
    bondingDuration,
    currentEra,
    freeBalanceOf,
    intentions,
    sessionsPerEra,
    transactionBaseFee,
    transactionByteFee,
    validatorCount
  }
}) => ({
  bondingDuration: (0, _bn.default)(db, bondingDuration, BLOCKNUM_SIZE),
  currentEra: (0, _bn.default)(db, currentEra, BLOCKNUM_SIZE),
  freeBalanceOf: (0, _bn.default)(db, freeBalanceOf, BALANCE_SIZE),
  intentions: (0, _arrayAccount.default)(db, intentions),
  sessionsPerEra: (0, _bn.default)(db, sessionsPerEra, BLOCKNUM_SIZE),
  transactionBaseFee: (0, _bn.default)(db, transactionBaseFee, BALANCE_SIZE),
  transactionByteFee: (0, _bn.default)(db, transactionByteFee, BALANCE_SIZE),
  validatorCount: (0, _bn.default)(db, validatorCount, 32)
});

const system = (db, {
  public: {
    accountIndexOf,
    blockHashAt
  }
}) => ({
  accountIndexOf: (0, _bn.default)(db, accountIndexOf, 32),
  blockHashAt: (0, _u8a.default)(db, blockHashAt)
});

const timestamp = (db, {
  public: {
    didUpdate
  }
}) => ({
  didUpdate: (0, _bool.default)(db, didUpdate)
});

function createState(db) {
  return {
    db,
    consensus: consensus(db, _storage.default.consensus),
    council: council(db, _storage.default.council),
    councilVoting: councilVoting(db, _storage.default.councilVoting),
    democracy: democracy(db, _storage.default.democracy),
    parachains: parachains(db, _storage.default.parachains),
    session: session(db, _storage.default.session),
    staking: staking(db, _storage.default.staking),
    system: system(db, _storage.default.system),
    timestamp: timestamp(db, _storage.default.timestamp)
  };
}